<?php

/**
 * Copyright (C) VasylTech, LLC <vasyl@vasyltech.com>
 *
 * This program is commercial software: you are not allowed to redistribute it
 * and/or modify. Unauthorized copying of this file, via any medium is strictly
 * prohibited.
 * For any questions or concerns contact Vasyl Martyniuk <vasyl@vasyltech.com>
 */

/**
 * AAM RESTful Term Resource
 * 
 * @package AAM
 * @author Vasyl Martyniuk <vasyl@vasyltech.com>
 */
class AAM_PlusPackage_Api_Rest_Resource_Term {
    
    /**
     * Instance of itself
     * 
     * @var AAM_PlusPackage_Api_Rest_Resource_Term
     * 
     * @access private 
     */
    private static $_instance = null;
    
    /**
     * Authorize Post actions
     * 
     * @param WP_REST_Request $request
     * 
     * @return WP_Error|null
     * 
     * @access public
     */
    public function authorize($request) {
        $response = null;
        
        if ($request['id']) {
            // Add trailing | to keep it compatible with AAM core standard
            $term = AAM::api()->getUser()->getObject('term', $request['id'] . '|');
        
            switch($request->get_method()) {
                case 'GET':
                    $response = $this->checkBrowseAuth($term);
                    break;

                case 'POST':
                case 'PUT':
                case 'PATCH':
                    $response = $this->checkUpdateAuth($term);
                    break;

                case 'DELETE':
                    $response = $this->checkDeleteAuth($term);
                    break;

                default:
                    break;
            }
        }
        
        return $response;
    }
    
    /**
     * 
     * @param AAM_Object_Term $term
     * @return type
     */
    protected function checkBrowseAuth(AAM_Object_Term $term) {
        $result = null;
        
        if ($term->has('term', 'api.browse')) {
            $result = new WP_Error(
                'rest_browse_access_denied', 
                "You are not allowed to browse this term. Access denied",
                array(
                    'status' => 403
                )
            );
        }
        
        return $result;
    }
    
    /**
     * 
     * @param AAM_Object_Term $term
     * @return type
     */
    protected function checkUpdateAuth(AAM_Object_Term $term) {
        $result = null;
        
        if ($term->has('term', 'api.edit')) {
            $result = new WP_Error(
                'rest_edit_denied', 
                "You are not allowed to edit this term. Access denied",
                array(
                    'status' => 403
                )
            );
        }
        
        return $result;
    }
    
    /**
     * 
     * @param AAM_Object_Term $term
     * @return type
     */
    protected function checkDeleteAuth(AAM_Object_Term $term) {
        $result = null;
        
        if ($term->has('term', 'api.delete')) {
            $result = new WP_Error(
                'rest_delete_denied', 
                "You are not allowed to delete this term. Access denied",
                array(
                    'status' => 403
                )
            );
        }
        
        return $result;
    }
    
    /**
     * Alias for the bootstrap
     * 
     * @return AAM_PlusPackage_Api_Rest_Resource_Term
     * 
     * @access public
     * @static
     */
    public static function getInstance() {
        return self::bootstrap();
    }
    
    /**
     * Bootstrap authorization layer
     * 
     * @return AAM_PlusPackage_Api_Rest_Resource_Term
     * 
     * @access public
     */
    public static function bootstrap() {
        if (is_null(self::$_instance)) {
            self::$_instance = new self;
        }
        
        return self::$_instance;
    }
}