<?php

/**
 * Copyright (C) VasylTech, LLC <vasyl@vasyltech.com>
 *
 * This program is commercial software: you are not allowed to redistribute it
 * and/or modify. Unauthorized copying of this file, via any medium is strictly
 * prohibited.
 * For any questions or concerns contact Vasyl Martyniuk <vasyl@vasyltech.com>
 */

/**
 * AAM Plus Package frontend
 *
 * @package AAM
 * @author Vasyl Martyniuk <vasyl@vasyltech.com>
 */
class AAM_PlusPackage_Frontend {

    /**
     * Instance of itself
     * 
     * @var AAM_PlusPackage_Frontend 
     * 
     * @access private
     */
    private static $_instance = null;

    /**
     * Initialize the extension
     * 
     * @return void
     * 
     * @access protected
     */
    protected function __construct() {
        //navigation menu
        add_filter('wp_get_nav_menu_items', array($this, 'getNavigationMenu'));
        //control WordPress frontend
        add_action('wp', array($this, 'wp'), 999);
    }

    /**
     * Main Frontend access control hook
     *
     * @return void
     *
     * @access public
     * @global WP_Query $wp_query
     */
    public function wp() {
        global $wp_query;
        
        if (is_category() || is_archive()) {
            $term = $wp_query->get_queried_object();
            
            if (is_a($term, 'WP_Term') 
                    && $this->checkTerm($term->term_id, $term->taxonomy, 'browse')) {
                AAM_Core_API::reject(
                    'frontend', 
                    array(
                        'hook'   => 'term_browse', 
                        'action' => 'frontend.browse', 
                        'term'   => $term
                    )
                );
            }
        }
    }
    
    /**
     * Filter Navigation menu
     *
     * @param array $pages
     *
     * @return array
     *
     * @access public
     */
    public function getNavigationMenu($pages) {
        foreach ((is_array($pages) ? $pages : array()) as $i => $page) {
            if ($page->type === 'taxonomy') {
                if ($this->checkTerm($page->object_id, $page->object, 'list')) {
                    unset($pages[$i]);
                }
            }
        }

        return $pages;
    }

    /**
     * Check if term need to be filtered
     * 
     * @param string $id
     * @param string $taxonomy
     * @param string $action
     * 
     * @return boolean
     * 
     * @access protected
     */
    protected function checkTerm($id, $taxonomy, $action) {
        $response = false;
        $object   = AAM_PlusPackage_Base::getInstance()->initTermAccessSettings(
                $id, $taxonomy
        );
        
        if ($object->has('term', 'frontend.' . $action)) {
            $response = true;
        }
        
        return $response;
    }
    
    /**
     * Bootstrap the extension
     * 
     * @return AAM_PlusPackage_Frontend
     * 
     * @access public
     */
    public static function bootstrap() {
        if (is_null(self::$_instance)) {
            self::$_instance = new self;
        }

        return self::$_instance;
    }

}